# Copyright 2019 - China Mobile (SuZhou) Software Technology Co.,Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class StackProperties(object):
    STACK_NAME = 'stack_name'
    STACK_STATUS = 'stack_status'
    ID = 'id'
    STATE = 'state'
    CREATED_AT = 'created_at'
    STACK_IDENTITY = 'stack_identity'
    CREATION_TIME = 'creation_time'
    PROJECT = 'project'
    RESOURCES = 'resources'
    RESOURCES_TYPE = 'resource_type'
    PHYSICAL_RESOURCE_ID = 'physical_resource_id'
    TENANT_ID = 'tenant_id'
