# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

ADD_VERTEX = 'add_vertex'
REMOVE_VERTEX = 'remove_vertex'
UPDATE_VERTEX = 'update_vertex'
ADD_EDGE = 'add_edge'
REMOVE_EDGE = 'remove_edge'
EXECUTE_EXTERNAL = 'execute_external'

EXECUTION_ENGINE = 'execution_engine'
