# Copyright 2016 - Nokia Corporation
# Copyright 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# noinspection PyPackageRequirements

from vitrage.tests.functional import api


class HealthTest(api.FunctionalTest):

    def __init__(self, *args, **kwds):
        super(HealthTest, self).__init__(*args, **kwds)
        self.auth = 'keystone'

    def test_health(self):
        resp = self.app.get('/healthcheck')
        self.assertEqual('200 OK', resp.status)

    def test_health_json(self):
        resp = self.app.get('/healthcheck',
                            headers={'Accept': 'application/json'})
        self.assertEqual('200 OK', resp.status)
        self.assertDictEqual({'detailed': False, 'reasons': ['OK']},
                             resp.json, None)
