#  Copyright 2019 - Nokia Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_policy import policy

from vitrage.common.policies import base

SERVICE = 'get service list'

rules = [
    policy.DocumentedRuleDefault(
        name=SERVICE,
        check_str=base.UNPROTECTED,
        description='Get the vitrage services for the OpenStack cluster',
        operations=[
            {
                'path': '/service',
                'method': 'GET'
            }
        ]
    )
]


def list_rules():
    return rules
